@@ -684,13 +684,13 @@ def master_callback(m, master):
 
     elif mtype == "GPS_RAW_INT":
         if mpstate.status.have_gps_lock:
-            if m.fix_type != 3 and not mpstate.status.lost_gps_lock and (time.time() - mpstate.status.last_gps_lock) > 3:
+            if m.fix_type < 3 and not mpstate.status.lost_gps_lock and (time.time() - mpstate.status.last_gps_lock) > 3:
                 say("GPS fix lost")
                 mpstate.status.lost_gps_lock = True
-            if m.fix_type == 3 and mpstate.status.lost_gps_lock:
+            if m.fix_type >= 3 and mpstate.status.lost_gps_lock:
                 say("GPS OK")
                 mpstate.status.lost_gps_lock = False
-            if m.fix_type == 3:
+            if m.fix_type >= 3:
                 mpstate.status.last_gps_lock = time.time()
 
     elif mtype == "NAV_CONTROLLER_OUTPUT" and mpstate.status.flightmode == "AUTO" and mpstate.settings.distreadout:
@@ -959,6 +959,7 @@ def main_loop():
             return
         while not mpstate.input_queue.empty():
             line = mpstate.input_queue.get()
+            mpstate.input_count += 1
             cmds = line.split(';')
             for c in cmds:
                 process_stdin(c)
@@ -1192,6 +1193,7 @@ Auto-detected serial ports are:
     heartbeat_check_period = mavutil.periodic_event(0.33)
 
     mpstate.input_queue = Queue.Queue()
+    mpstate.input_count = 0
     mpstate.rl = rline.rline("MAV> ", mpstate)
     if opts.setup:
         mpstate.rl.set_prompt("")
@@ -1211,7 +1213,7 @@ Auto-detected serial ports are:
     if not opts.setup:
         # some core functionality is in modules
         standard_modules = ['log', 'wp', 'rally','fence','param','relay',
-                            'tuneopt','arm','mode','calibration','rc','auxopt','misc']
+                            'tuneopt','arm','mode','calibration','rc','auxopt','misc','cmdlong']
         for m in standard_modules:
             load_module(m, quiet=True)
 
