@@ -20,15 +20,16 @@ from MAVProxy.modules.mavproxy_map import mp_elevation
 from MAVProxy.modules.mavproxy_map import mp_tile
 from MAVProxy.modules.lib import mp_util
 from MAVProxy.modules.lib.mp_menu import *
-from MAVProxy.modules.mavproxy_map import mp_widgets
+from MAVProxy.modules.lib import mp_widgets
 
 
 class SlipObject:
     '''an object to display on the map'''
-    def __init__(self, key, layer):
+    def __init__(self, key, layer, popup_menu=None):
         self.key = key
         self.layer = layer
         self.latlon = None
+        self.popup_menu = popup_menu
 
     def clip(self, px, py, w, h, img):
         '''clip an area for display on the map'''
@@ -68,14 +69,20 @@ class SlipObject:
         '''
         return None
 
+    def selection_info(self):
+        '''extra selection information sent when object is selected'''
+        return None
+
 class SlipPolygon(SlipObject):
     '''a polygon to display on the map'''
-    def __init__(self, key, points, layer, colour, linewidth):
-        SlipObject.__init__(self, key, layer, )
+    def __init__(self, key, points, layer, colour, linewidth, popup_menu=None):
+        SlipObject.__init__(self, key, layer, popup_menu=popup_menu)
         self.points = points
         self.colour = colour
         self.linewidth = linewidth
         self._bounds = mp_util.polygon_bounds(self.points)
+        self._pix_points = []
+        self._selected_vertex = None
 
     def bounds(self):
         '''return bounding box'''
@@ -91,9 +98,13 @@ class SlipPolygon(SlipObject):
         (pix1, pix2) = clipped
         cv.Line(img, pix1, pix2, colour, linewidth)
         cv.Circle(img, pix2, linewidth*2, colour)
+        if len(self._pix_points) == 0:
+            self._pix_points.append(pix1)
+        self._pix_points.append(pix2)
 
     def draw(self, img, pixmapper, bounds):
         '''draw a polygon on the image'''
+        self._pix_points = []
         for i in range(len(self.points)-1):
             if len(self.points[i]) > 2:
                 colour = self.points[i][2]
@@ -102,6 +113,21 @@ class SlipPolygon(SlipObject):
             self.draw_line(img, pixmapper, self.points[i], self.points[i+1],
                            colour, self.linewidth)
 
+    def clicked(self, px, py):
+        '''see if the polygon has been clicked on.
+        Consider it clicked if the pixel is within 6 of the point
+        '''
+        for i in range(len(self._pix_points)):
+            (pixx,pixy) = self._pix_points[i]
+            if abs(px - pixx) < 6 and abs(py - pixy) < 6:
+                self._selected_vertex = i
+                return math.sqrt((px - pixx)**2 + (py - pixy)**2)
+        return None
+
+    def selection_info(self):
+        '''extra selection information sent when object is selected'''
+        return self._selected_vertex
+
 
 class SlipGrid(SlipObject):
     '''a map grid'''
@@ -154,8 +180,9 @@ class SlipGrid(SlipObject):
 class SlipThumbnail(SlipObject):
     '''a thumbnail to display on the map'''
     def __init__(self, key, latlon, layer, img,
-                 border_colour=None, border_width=0):
-        SlipObject.__init__(self, key, layer)
+                 border_colour=None, border_width=0,
+                 popup_menu=None):
+        SlipObject.__init__(self, key, layer, popup_menu=popup_menu)
         self.latlon = latlon
         self._img = None
         self.imgstr = img.tostring()
@@ -242,8 +269,8 @@ class SlipTrail:
 class SlipIcon(SlipThumbnail):
     '''a icon to display on the map'''
     def __init__(self, key, latlon, img, layer=1, rotation=0,
-                 follow=False, trail=None):
-        SlipThumbnail.__init__(self, key, latlon, layer, img)
+                 follow=False, trail=None, popup_menu=None):
+        SlipThumbnail.__init__(self, key, latlon, layer, img, popup_menu=popup_menu)
         self.rotation = rotation
         self.follow = follow
         self.trail = trail
@@ -333,6 +360,11 @@ class SlipInformation:
         '''update the information'''
         pass
 
+class SlipDefaultPopup:
+    '''an object to hold a default popup menu'''
+    def __init__(self, popup, combine=False):
+        self.popup = popup
+        self.combine = combine
 
 class SlipInfoImage(SlipInformation):
     '''an image to display in the info box'''
@@ -404,9 +436,11 @@ class SlipInfoText(SlipInformation):
 
 class SlipObjectSelection:
     '''description of a object under the cursor during an event'''
-    def __init__(self, objkey, distance):
+    def __init__(self, objkey, distance, layer, extra_info=None):
         self.distance = distance
         self.objkey = objkey
+        self.layer = layer
+        self.extra_info = extra_info
 
 class SlipEvent:
     '''an event sent to the parent.
@@ -430,6 +464,12 @@ class SlipKeyEvent(SlipEvent):
     def __init__(self, latlon, event, selected):
         SlipEvent.__init__(self, latlon, event, selected)
 
+class SlipMenuEvent(SlipEvent):
+    '''a menu event sent to the parent'''
+    def __init__(self, latlon, event, selected, menuitem):
+        SlipEvent.__init__(self, latlon, event, selected)
+        self.menuitem = menuitem
+
 
 class MPSlipMap():
     '''
@@ -554,6 +594,10 @@ class MPSlipMapFrame(wx.Frame):
         state.frame = self
         state.grid = True
         state.follow = True
+        state.popup_object = None
+        state.popup_latlon = None
+        state.popup_started = False
+        state.default_popup = None
         state.panel = MPSlipMapPanel(self, state)
         self.Bind(wx.EVT_IDLE, self.on_idle)
         self.Bind(wx.EVT_SIZE, state.panel.on_size)
@@ -577,9 +621,28 @@ class MPSlipMapFrame(wx.Frame):
     def on_menu(self, event):
         '''handle menu selection'''
         state = self.state
+        # see if it is a popup menu
+        if state.popup_object is not None:
+            obj = state.popup_object
+            ret = obj.popup_menu.find_selected(event)
+            if ret is not None:
+                ret.call_handler()
+                state.event_queue.put(SlipMenuEvent(state.popup_latlon, event,
+                                                    [SlipObjectSelection(obj.key, 0, obj.layer, obj.selection_info())],
+                                                    ret))
+                state.popup_object = None
+                state.popup_latlon = None
+        if state.default_popup is not None:
+            ret = state.default_popup.popup.find_selected(event)
+            if ret is not None:
+                ret.call_handler()
+                state.event_queue.put(SlipMenuEvent(state.popup_latlon, event, [], ret))
+            
+        # otherwise a normal menu
         ret = self.menu.find_selected(event)
         if ret is None:
             return
+        ret.call_handler()
         if ret.returnkey == 'toggleGrid':
             state.grid = ret.IsChecked()
         elif ret.returnkey == 'toggleFollow':
@@ -662,6 +725,9 @@ class MPSlipMapFrame(wx.Frame):
                         self.follow(object)
                     state.need_redraw = True
 
+            if isinstance(obj, SlipDefaultPopup):
+                state.default_popup = obj
+
             if isinstance(obj, SlipInformation):
                 # see if its a existing one or a new one
                 if obj.key in state.info:
@@ -949,12 +1015,32 @@ class MPSlipMapPanel(wx.Panel):
         (px, py) = pos
         for layer in state.layers:
             for key in state.layers[layer]:
-                distance = state.layers[layer][key].clicked(px, py)
+                obj = state.layers[layer][key]
+                distance = obj.clicked(px, py)
                 if distance is not None:
-                    selected.append(SlipObjectSelection(key, distance))
+                    selected.append(SlipObjectSelection(key, distance, layer, extra_info=obj.selection_info()))
         selected.sort(key=lambda c: c.distance)
         return selected
 
+    def show_popup(self, selected, pos):
+        '''show popup menu for an object'''
+        state = self.state
+        if selected.popup_menu is not None:
+            import copy
+            popup_menu = selected.popup_menu
+            if state.default_popup.popup is not None and state.default_popup.combine:
+                popup_menu = copy.deepcopy(popup_menu)
+                popup_menu.add(MPMenuSeparator())
+                popup_menu.combine(state.default_popup.popup)
+            wx_menu = popup_menu.wx_menu()
+            state.frame.PopupMenu(wx_menu, pos)
+
+    def show_default_popup(self, pos):
+        '''show default popup menu'''
+        state = self.state
+        if state.default_popup.popup is not None:
+            wx_menu = state.default_popup.popup.wx_menu()
+            state.frame.PopupMenu(wx_menu, pos)
 
     def on_mouse(self, event):
         '''handle mouse events'''
@@ -971,8 +1057,25 @@ class MPSlipMapPanel(wx.Panel):
             latlon = self.coordinates(pos.x, pos.y)
             selected = self.selected_objects(pos)
             state.event_queue.put(SlipMouseEvent(latlon, event, selected))
+            if event.RightDown():
+                state.popup_object = None
+                state.popup_latlon = None
+                if len(selected) > 0:
+                    obj = state.layers[selected[0].layer][selected[0].objkey]
+                    if obj.popup_menu is not None:
+                        state.popup_object = obj
+                        state.popup_latlon = latlon
+                        self.show_popup(obj, pos)
+                        state.popup_started = True
+                if not state.popup_started and state.default_popup is not None:
+                    state.popup_latlon = latlon
+                    self.show_default_popup(pos)
+                    state.popup_started = True
 
-        if event.LeftDown():
+        if not event.ButtonIsDown(wx.MOUSE_BTN_RIGHT):
+            state.popup_started = False
+
+        if event.LeftDown() or event.RightDown():
             self.mouse_down = pos
             self.last_click_pos = self.click_pos
             self.click_pos = self.coordinates(pos.x, pos.y)
@@ -992,6 +1095,7 @@ class MPSlipMapPanel(wx.Panel):
                 self.redraw_map()
 
     def clear_thumbnails(self):
+        '''clear all thumbnails from the map'''
         state = self.state
         for l in state.layers:
             keys = state.layers[l].keys()[:]
